@@ -1,6 +1,6 @@
 use crate::actor::Actor;
 use bevy::prelude::*;
-use std::collections::HashSet;
+use std::{collections::HashSet, hash::Hash};
 
 pub struct PhysicsPlugin;
 
@@ -38,8 +38,25 @@ impl CollisionState {
     }
 }
 
-#[derive(Debug, Default, Eq, Hash, Clone)]
-pub struct CollisionPair(String, String);
+#[derive(Debug, Default, Eq, Clone)]
+pub struct CollisionPair(pub String, pub String);
+
+impl CollisionPair {
+    pub fn either_contains<T: Into<String>>(&self, label: T) -> bool {
+        let label = label.into();
+        (self.0 == label) || (self.1 == label)
+    }
+    pub fn either_starts_with<T: Into<String>>(&self, label: T) -> bool {
+        let label = label.into();
+        self.0.starts_with(&label) || self.1.starts_with(&label)
+    }
+    pub fn one_starts_with<T: Into<String>>(&self, label: T) -> bool {
+        let label = label.into();
+        let a_matches = self.0.starts_with(&label);
+        let b_matches = self.1.starts_with(&label);
+        (a_matches && !b_matches) || (!a_matches && b_matches)
+    }
+}
 
 impl PartialEq for CollisionPair {
     fn eq(&self, other: &Self) -> bool {
@@ -47,10 +64,17 @@ impl PartialEq for CollisionPair {
     }
 }
 
-impl CollisionPair {
-    pub fn contains<T: Into<String>>(&self, label: T) -> bool {
-        let label = label.into();
-        (self.0 == label) || (self.1 == label)
+impl Hash for CollisionPair {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        // Make sure we return the same hash no matter which position the same two strings might be
+        // in (so we match our PartialEq implementation)
+        if self.0 < self.1 {
+            self.0.hash(state);
+            self.1.hash(state);
+        } else {
+            self.1.hash(state);
+            self.0.hash(state);
+        }
     }
 }
 
@@ -66,7 +90,7 @@ fn collision_detection(
                 // We only need to compare one half of the matrix triangle
                 continue 'outer;
             }
-            if Collider::colliding(&actor1, &actor2) {
+            if Collider::colliding(actor1, actor2) {
                 current_collisions
                     .insert(CollisionPair(actor1.label.clone(), actor2.label.clone()));
             }
@@ -75,7 +99,7 @@ fn collision_detection(
 
     let beginning_collisions: Vec<_> = current_collisions
         .difference(&existing_collisions)
-        .map(|x| x.clone())
+        .cloned()
         .collect();
 
     collision_events.send_batch(beginning_collisions.iter().map(|p| CollisionEvent {
@@ -89,7 +113,7 @@ fn collision_detection(
 
     let ending_collisions: Vec<_> = existing_collisions
         .difference(&current_collisions)
-        .map(|x| x.clone())
+        .cloned()
         .collect();
 
     collision_events.send_batch(ending_collisions.iter().map(|p| CollisionEvent {
@@ -126,7 +150,7 @@ impl Collider {
         ])
     }
     pub fn poly<T: Into<Vec2> + Copy>(points: &[T]) -> Self {
-        Self::Poly(points.into_iter().map(|&x| x.into()).collect())
+        Self::Poly(points.iter().map(|&x| x.into()).collect())
     }
     pub fn circle_custom(radius: f32, vertices: usize) -> Self {
         let mut points = vec![];
@@ -143,10 +167,7 @@ impl Collider {
         Self::circle_custom(radius, 16)
     }
     pub fn is_poly(&self) -> bool {
-        match self {
-            Self::Poly(_) => true,
-            _ => false,
-        }
+        matches!(self, Self::Poly(_))
     }
     fn rotated(&self, rotation: f32) -> Vec<Vec2> {
         let mut rotated_points = Vec::new();
@@ -181,7 +202,7 @@ impl Collider {
             let poly2 = actor2.collider.relative_to(actor2);
             // Polygon intersection algorithm adapted from
             // https://stackoverflow.com/questions/10962379/how-to-check-intersection-between-2-rotated-rectangles
-            for poly in [poly1.clone(), poly2.clone()] {
+            for poly in vec![poly1.clone(), poly2.clone()] {
                 for (idx, &p1) in poly.iter().enumerate() {
                     let p2 = poly[(idx + 1) % poly.len()];
                     let normal = Vec2::new(p2.y - p1.y, p1.x - p2.x);
